@@ -70,7 +70,9 @@
 #include <cstddef>
 #include <cstring>
 #include <ctime>
+#include <iomanip>
 #include <regex>
+#include <sstream>
 #include <sys/stat.h>
 #include "goo/glibc.h"
 #include "goo/gstrtod.h"
@@ -139,7 +141,7 @@ PDFDoc::PDFDoc(std::unique_ptr<GooString> &&fileNameA, const std::optional<GooSt
     file = GooFile::open(fileName->toStr());
 #endif
 
-    if (file == nullptr) {
+    if (!file) {
         // fopen() has failed.
         // Keep a copy of the errno returned by fopen so that it can be
         // referred to later.
@@ -150,7 +152,7 @@ PDFDoc::PDFDoc(std::unique_ptr<GooString> &&fileNameA, const std::optional<GooSt
     }
 
     // create stream
-    str = new FileStream(file, 0, false, file->size(), Object(objNull));
+    str = new FileStream(file.get(), 0, false, file->size(), Object(objNull));
 
     ok = setup(ownerPassword, userPassword, xrefReconstructedCallback);
 }
@@ -186,7 +188,7 @@ PDFDoc::PDFDoc(wchar_t *fileNameA, int fileNameLen, const std::optional<GooStrin
     }
 
     // create stream
-    str = new FileStream(file, 0, false, file->size(), Object(objNull));
+    str = new FileStream(file.get(), 0, false, file->size(), Object(objNull));
 
     ok = setup(ownerPassword, userPassword, xrefReconstructedCallback);
 }
@@ -300,7 +302,6 @@ PDFDoc::~PDFDoc()
     delete hints;
     delete linearization;
     delete str;
-    delete file;
 #ifdef _WIN32
     gfree(fileNameU);
 #endif
@@ -316,8 +317,9 @@ bool PDFDoc::checkFooter()
     int i, ch;
     for (i = 0; i < 1024; i++) {
         ch = str->getChar();
-        if (ch == EOF)
+        if (ch == EOF) {
             break;
+        }
         eof[i] = ch;
     }
     eof[i] = '\0';
@@ -356,8 +358,9 @@ void PDFDoc::checkHeader()
     // read up to headerSearchSize bytes from the beginning of the document
     for (i = 0; i < headerSearchSize; ++i) {
         const int c = str->getChar();
-        if (c == EOF)
+        if (c == EOF) {
             break;
+        }
         hdrBuf[i] = c;
     }
     bytesRead = i;
@@ -561,8 +564,9 @@ std::vector<FormFieldSignature *> PDFDoc::getSignatureFields()
 
     // First search
     const Form *f = catalog->getForm();
-    if (!f)
+    if (!f) {
         return res;
+    }
 
     const int nRootFields = f->getNumFields();
     for (int i = 0; i < nRootFields; ++i) {
@@ -598,8 +602,9 @@ void PDFDoc::displayPage(OutputDev *out, int page, double hDPI, double vDPI, int
         printf("***** page %d *****\n", page);
     }
 
-    if (getPage(page))
+    if (getPage(page)) {
         getPage(page)->display(out, hDPI, vDPI, rotate, useMediaBox, crop, printing, abortCheckCbk, abortCheckCbkData, annotDisplayDecideCbk, annotDisplayDecideCbkData, copyXRef);
+    }
 }
 
 void PDFDoc::displayPages(OutputDev *out, int firstPage, int lastPage, double hDPI, double vDPI, int rotate, bool useMediaBox, bool crop, bool printing, bool (*abortCheckCbk)(void *data), void *abortCheckCbkData,
@@ -615,8 +620,9 @@ void PDFDoc::displayPages(OutputDev *out, int firstPage, int lastPage, double hD
 void PDFDoc::displayPageSlice(OutputDev *out, int page, double hDPI, double vDPI, int rotate, bool useMediaBox, bool crop, bool printing, int sliceX, int sliceY, int sliceW, int sliceH, bool (*abortCheckCbk)(void *data),
                               void *abortCheckCbkData, bool (*annotDisplayDecideCbk)(Annot *annot, void *user_data), void *annotDisplayDecideCbkData, bool copyXRef)
 {
-    if (getPage(page))
+    if (getPage(page)) {
         getPage(page)->displaySlice(out, hDPI, vDPI, rotate, useMediaBox, crop, sliceX, sliceY, sliceW, sliceH, printing, abortCheckCbk, abortCheckCbkData, annotDisplayDecideCbk, annotDisplayDecideCbkData, copyXRef);
+    }
 }
 
 std::unique_ptr<Links> PDFDoc::getLinks(int page)
@@ -630,8 +636,9 @@ std::unique_ptr<Links> PDFDoc::getLinks(int page)
 
 void PDFDoc::processLinks(OutputDev *out, int page)
 {
-    if (getPage(page))
+    if (getPage(page)) {
         getPage(page)->processLinks(out);
+    }
 }
 
 Linearization *PDFDoc::getLinearization()
@@ -645,12 +652,15 @@ Linearization *PDFDoc::getLinearization()
 
 bool PDFDoc::checkLinearization()
 {
-    if (linearization == nullptr)
+    if (linearization == nullptr) {
         return false;
-    if (linearizationState == 1)
+    }
+    if (linearizationState == 1) {
         return true;
-    if (linearizationState == 2)
+    }
+    if (linearizationState == 2) {
         return false;
+    }
     if (!hints) {
         hints = new Hints(str, linearization, getXRef(), secHdlr);
     }
@@ -686,13 +696,14 @@ bool PDFDoc::checkLinearization()
 
 bool PDFDoc::isLinearized(bool tryingToReconstruct)
 {
-    if ((str->getLength()) && (getLinearization()->getLength() == str->getLength()))
+    if ((str->getLength()) && (getLinearization()->getLength() == str->getLength())) {
         return true;
-    else {
-        if (tryingToReconstruct)
+    } else {
+        if (tryingToReconstruct) {
             return getLinearization()->getLength() > 0;
-        else
+        } else {
             return false;
+        }
     }
 }
 
@@ -746,13 +757,15 @@ static bool get_id(const GooString *encodedidstring, GooString *id)
     char pdfid[pdfIdLength + 1];
     int n;
 
-    if (encodedidstring->getLength() != pdfIdLength / 2)
+    if (encodedidstring->getLength() != pdfIdLength / 2) {
         return false;
+    }
 
     n = sprintf(pdfid, "%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x", encodedid[0] & 0xff, encodedid[1] & 0xff, encodedid[2] & 0xff, encodedid[3] & 0xff, encodedid[4] & 0xff, encodedid[5] & 0xff, encodedid[6] & 0xff,
                 encodedid[7] & 0xff, encodedid[8] & 0xff, encodedid[9] & 0xff, encodedid[10] & 0xff, encodedid[11] & 0xff, encodedid[12] & 0xff, encodedid[13] & 0xff, encodedid[14] & 0xff, encodedid[15] & 0xff);
-    if (n != pdfIdLength)
+    if (n != pdfIdLength) {
         return false;
+    }
 
     id->Set(pdfid, pdfIdLength);
     return true;
@@ -808,8 +821,9 @@ int PDFDoc::savePageAs(const GooString &name, int pageNo)
     OutStream *outStr;
     XRef *yRef, *countRef;
 
-    if (file && file->modificationTimeChangedSinceOpen())
+    if (file && file->modificationTimeChangedSinceOpen()) {
         return errFileChangedSinceOpen;
+    }
 
     int rootNum = getXRef()->getNumObjects() + 1;
 
@@ -880,8 +894,9 @@ int PDFDoc::savePageAs(const GooString &name, int pageNo)
     }
     Dict *pagesDict = pagesObj.getDict();
     Object resourcesObj = pagesDict->lookup("Resources");
-    if (resourcesObj.isDict())
+    if (resourcesObj.isDict()) {
         markPageObjects(resourcesObj.getDict(), yRef, countRef, 0, refPage->num, rootNum + 2);
+    }
     markPageObjects(catDict, yRef, countRef, 0, refPage->num, rootNum + 2);
 
     Dict *pageDict = page.getDict();
@@ -906,8 +921,9 @@ int PDFDoc::savePageAs(const GooString &name, int pageNo)
     for (int j = 0; j < catDict->getLength(); j++) {
         const char *key = catDict->getKey(j);
         if (strcmp(key, "Type") != 0 && strcmp(key, "Catalog") != 0 && strcmp(key, "Pages") != 0) {
-            if (j > 0)
+            if (j > 0) {
                 outStr->printf(" ");
+            }
             Object value = catDict->getValNF(j).copy();
             outStr->printf("/%s ", key);
             writeObject(&value, outStr, getXRef(), 0, nullptr, cryptRC4, 0, 0, 0);
@@ -929,8 +945,9 @@ int PDFDoc::savePageAs(const GooString &name, int pageNo)
     outStr->printf("%d 0 obj\n", rootNum + 2);
     outStr->printf("<< ");
     for (int n = 0; n < pageDict->getLength(); n++) {
-        if (n > 0)
+        if (n > 0) {
             outStr->printf(" ");
+        }
         const char *key = pageDict->getKey(n);
         Object value = pageDict->getValNF(n).copy();
         if (strcmp(key, "Parent") == 0) {
@@ -977,8 +994,9 @@ int PDFDoc::saveAs(const GooString &name, PDFWriteMode mode)
 
 int PDFDoc::saveAs(OutStream *outStr, PDFWriteMode mode)
 {
-    if (file && file->modificationTimeChangedSinceOpen())
+    if (file && file->modificationTimeChangedSinceOpen()) {
         return errFileChangedSinceOpen;
+    }
 
     if (!xref->isModified() && mode == writeStandard) {
         // simply copy the original file
@@ -1016,8 +1034,9 @@ int PDFDoc::saveWithoutChangesAs(OutStream *outStr)
 {
     int c;
 
-    if (file && file->modificationTimeChangedSinceOpen())
+    if (file && file->modificationTimeChangedSinceOpen()) {
         return errFileChangedSinceOpen;
+    }
 
     BaseStream *copyStr = str->copy();
     copyStr->reset();
@@ -1052,8 +1071,9 @@ void PDFDoc::saveIncrementalUpdate(OutStream *outStr)
     uxref->add(0, 65535, 0, false);
     xref->lock();
     for (int i = 0; i < xref->getNumObjects(); i++) {
-        if ((xref->getEntry(i)->type == xrefEntryFree) && (xref->getEntry(i)->gen == 0)) // we skip the irrelevant free objects
+        if ((xref->getEntry(i)->type == xrefEntryFree) && (xref->getEntry(i)->gen == 0)) { // we skip the irrelevant free objects
             continue;
+        }
 
         if (xref->getEntry(i)->getFlag(XRefEntry::Updated)) { // we have an updated object
             Ref ref;
@@ -1128,8 +1148,9 @@ void PDFDoc::saveCompleteRewrite(OutStream *outStr)
             ref.gen = xref->getEntry(i)->gen;
             /* the XRef class adds a lot of irrelevant free entries, we only want the significant one
                 and we don't want the one with num=0 because it has already been added (gen = 65535)*/
-            if (ref.gen > 0 && ref.num > 0)
+            if (ref.gen > 0 && ref.num > 0) {
                 uxref->add(ref, 0, false);
+            }
         } else if (xref->getEntry(i)->getFlag(XRefEntry::DontRewrite)) {
             // This entry must not be written, put a free entry instead (with incremented gen)
             ref.num = i;
@@ -1174,8 +1195,9 @@ void PDFDoc::writeDictionnary(Dict *dict, OutStream *outStr, XRef *xRef, unsigne
 
     if (alreadyWrittenDicts->find(dict) != alreadyWrittenDicts->end()) {
         error(errSyntaxWarning, -1, "PDFDoc::writeDictionnary: Found recursive dicts");
-        if (deleteSet)
+        if (deleteSet) {
             delete alreadyWrittenDicts;
+        }
         return;
     } else {
         alreadyWrittenDicts->insert(dict);
@@ -1216,10 +1238,11 @@ void PDFDoc::writeRawStream(Stream *str, OutStream *outStr)
     }
 
     Goffset length;
-    if (obj1.isInt())
+    if (obj1.isInt()) {
         length = obj1.getInt();
-    else
+    } else {
         length = obj1.getInt64();
+    }
 
     outStr->printf("stream\r\n");
     str->unfilteredReset();
@@ -1256,26 +1279,25 @@ void PDFDoc::writeString(const GooString *s, OutStream *outStr, const unsigned c
     if (s->hasUnicodeMarker()) {
         // unicode string don't necessary end with \0
         const char *c = s->c_str();
-        outStr->printf("(");
+        std::stringstream stream;
+        stream << std::setfill('0') << std::hex;
         for (int i = 0; i < s->getLength(); i++) {
-            char unescaped = *(c + i) & 0x000000ff;
-            // escape if needed
-            if (unescaped == '(' || unescaped == ')' || unescaped == '\\')
-                outStr->printf("%c", '\\');
-            outStr->printf("%c", unescaped);
+            stream << std::setw(2) << (0xff & (unsigned int)*(c + i));
         }
-        outStr->printf(") ");
+        outStr->printf("<");
+        outStr->printf("%s", stream.str().c_str());
+        outStr->printf("> ");
     } else {
         const char *c = s->c_str();
         outStr->printf("(");
         for (int i = 0; i < s->getLength(); i++) {
             char unescaped = *(c + i) & 0x000000ff;
             // escape if needed
-            if (unescaped == '\r')
+            if (unescaped == '\r') {
                 outStr->printf("\\r");
-            else if (unescaped == '\n')
+            } else if (unescaped == '\n') {
                 outStr->printf("\\n");
-            else {
+            } else {
                 if (unescaped == '(' || unescaped == ')' || unescaped == '\\') {
                     outStr->printf("%c", '\\');
                 }
@@ -1480,8 +1502,9 @@ Object PDFDoc::createTrailerDict(int uxrefSize, bool incrUpdate, Goffset startxR
     sprintf(buffer, "%i", (int)time(nullptr));
     message.append(buffer);
 
-    if (fileName)
+    if (fileName) {
         message.append(fileName);
+    }
 
     sprintf(buffer, "%lli", (long long)fileSize);
     message.append(buffer);
@@ -1518,7 +1541,9 @@ Object PDFDoc::createTrailerDict(int uxrefSize, bool incrUpdate, Goffset startxR
         // only update the second part of the array
         Object obj4 = xRef->getTrailerDict()->getDict()->lookup("ID");
         if (!obj4.isArray()) {
-            error(errSyntaxWarning, -1, "PDFDoc::createTrailerDict original file's ID entry isn't an array. Trying to continue");
+            if (hasEncrypt) {
+                error(errSyntaxWarning, -1, "PDFDoc::createTrailerDict original file's ID entry isn't an array. Trying to continue");
+            }
         } else {
             Array *array = new Array(xRef);
             // Get the first part of the ID
@@ -1613,8 +1638,9 @@ void PDFDoc::markDictionnary(Dict *dict, XRef *xRef, XRef *countRef, unsigned in
 
     if (alreadyMarkedDicts->find(dict) != alreadyMarkedDicts->end()) {
         error(errSyntaxWarning, -1, "PDFDoc::markDictionnary: Found recursive dicts");
-        if (deleteSet)
+        if (deleteSet) {
             delete alreadyMarkedDicts;
+        }
         return;
     } else {
         alreadyMarkedDicts->insert(dict);
@@ -1672,8 +1698,9 @@ void PDFDoc::markObject(Object *obj, XRef *xRef, XRef *countRef, unsigned int nu
         } else {
             XRefEntry *entry = countRef->getEntry(obj->getRef().num + numOffset);
             entry->gen++;
-            if (entry->gen > 9)
+            if (entry->gen > 9) {
                 break;
+            }
         }
         Object obj1 = getXRef()->fetch(obj->getRef());
         markObject(&obj1, xRef, countRef, numOffset, oldRefNum, newRefNum);
@@ -1946,8 +1973,9 @@ Goffset PDFDoc::getStartXRef(bool tryingToReconstruct)
                 if (!strncmp("endobj", &buf[i], 6)) {
                     i += 6;
                     // skip whitespace
-                    while (buf[i] && Lexer::isSpace(buf[i]))
+                    while (buf[i] && Lexer::isSpace(buf[i])) {
                         ++i;
+                    }
                     startXRefPos = i;
                     break;
                 }
@@ -1960,8 +1988,9 @@ Goffset PDFDoc::getStartXRef(bool tryingToReconstruct)
             // read last xrefSearchSize bytes
             int segnum = 0;
             int maxXRefSearch = 24576;
-            if (str->getLength() < maxXRefSearch)
-                maxXRefSearch = str->getLength();
+            if (str->getLength() < maxXRefSearch) {
+                maxXRefSearch = static_cast<int>(str->getLength());
+            }
             for (; (xrefSearchSize - 16) * segnum < maxXRefSearch; segnum++) {
                 str->setPos((xrefSearchSize - 16) * segnum + xrefSearchSize, -1);
                 for (n = 0; n < xrefSearchSize; ++n) {
@@ -1981,8 +2010,9 @@ Goffset PDFDoc::getStartXRef(bool tryingToReconstruct)
                 if (i < 0) {
                     startXRefPos = 0;
                 } else {
-                    for (p = &buf[i + 9]; isspace(*p); ++p)
+                    for (p = &buf[i + 9]; isspace(*p); ++p) {
                         ;
+                    }
                     startXRefPos = strToLongLong(p);
                     break;
                 }
@@ -2045,8 +2075,9 @@ Page *PDFDoc::parsePage(int page)
 
 Page *PDFDoc::getPage(int page)
 {
-    if ((page < 1) || page > getNumPages())
+    if ((page < 1) || page > getNumPages()) {
         return nullptr;
+    }
 
     if (isLinearized() && checkLinearization()) {
         pdfdocLocker();
@@ -2092,7 +2123,13 @@ bool PDFDoc::sign(const char *saveFilename, const char *certNickname, const char
         }
     }
 
-    const DefaultAppearance da { { objName, "SigFont" }, fontSize, std::move(fontColor) };
+    Form *form = catalog->getCreateForm();
+    std::string pdfFontName = form->findFontInDefaultResources("Helvetica", "");
+    if (pdfFontName.empty()) {
+        pdfFontName = form->addFontToDefaultResources("Helvetica", "");
+    }
+
+    const DefaultAppearance da { { objName, pdfFontName.c_str() }, fontSize, std::move(fontColor) };
 
     Object annotObj = Object(new Dict(getXRef()));
     annotObj.dictSet("Type", Object(objName, "Annot"));
@@ -2111,6 +2148,10 @@ bool PDFDoc::sign(const char *saveFilename, const char *certNickname, const char
 
     const Ref ref = getXRef()->addIndirectObject(annotObj);
     catalog->addFormToAcroForm(ref);
+    // say that there a now signatures and that we should append only
+    catalog->getAcroForm()->dictSet("SigFlags", Object(3));
+    form->ensureFontsForAllCharacters(&signatureText, pdfFontName);
+    form->ensureFontsForAllCharacters(&signatureTextLeft, pdfFontName);
 
     std::unique_ptr<::FormFieldSignature> field = std::make_unique<::FormFieldSignature>(this, Object(annotObj.getDict()), ref, nullptr, nullptr);
     field->setCustomAppearanceContent(signatureText);
